
package MooseX::AttributeHelpers::Counter;
use Moose;
use MooseX::AttributeHelpers::Sugar;

extends 'MooseX::AttributeHelpers::Base';

our $VERSION   = '0.03';
our $AUTHORITY = 'cpan:STEVAN';

define_attribute_helper (
    default_options  => {
        is      => 'ro', 
        default => 0,
    },

    helper_type      => 'Num',
    method_provider  => 'MooseX::AttributeHelpers::MethodProvider::Counter',
    auto_provide     => 1,
    shortcut         => 'Counter',
);

no Moose;
no MooseX::AttributeHelpers::Sugar;

1;

__END__

=pod

=head1 NAME

MooseX::AttributeHelpers::Counter

=head1 SYNOPSIS

  package MyHomePage;
  use Moose;
  use MooseX::AttributeHelpers;
  
  has 'counter' => (
      metaclass => 'Counter',
      is        => 'ro',
      isa       => 'Num',
      default   => sub { 0 },
      provides  => {
          inc => 'inc_counter',
          dec => 'dec_counter',          
          reset => 'reset_counter',
      }
  );

  my $page = MyHomePage->new();
  $page->inc_counter; # same as $page->counter($page->counter + 1);
  $page->dec_counter; # same as $page->counter($page->counter - 1);  
  
=head1 DESCRIPTION

This module provides a simple counter attribute, which can be incremented and 
decremented.  It is important to note that all those methods do in place 
modification of the value stored in the attribute.

If your attribute definition does not include any of I<is>, I<isa>,
I<default> or I<provides> but does use the C<Counter> metaclass,
then this module applies defaults as in the L</SYNOPSIS>
above. This allows for a very basic counter definition:

  has 'foo' => (metaclass => 'Counter');
  $obj->inc_foo;

=head1 PROVIDED METHODS

The methods for this metaclass are provided by
L<MooseX::AttributeHelpers::MethodProvider::Counter>.

=head1 BUGS

All complex software has bugs lurking in it, and this module is no 
exception. If you find a bug please either email me, or add the bug
to cpan-RT.

=head1 AUTHOR

Stevan Little E<lt>stevan@iinteractive.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2007-2008 by Infinity Interactive, Inc.

L<http://www.iinteractive.com>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
